/********************************** (C) COPYRIGHT *******************************
* File Name          : main.c
* Author             : WCH
* Version            : V1.0.0
* Date               : 2021/08/08
* Description        : Main program body.
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* SPDX-License-Identifier: Apache-2.0
*******************************************************************************/

/*
 *@Note
 RTC ¼ܹ MCU ͣģʽ»ѣΪʵִ˹ܣҪⲿж 17 ΪжϣҰ RTC óɿɲ¼
*/

#include "debug.h"

/* Global define */


/* Global Variable */

/* Exported_Functions */


void RTC17_NVIC_Config(void)
{
    EXTI_InitTypeDef EXTI_InitStruct = {0};

    EXTI_InitStruct.EXTI_Line = EXTI_Line17;
    EXTI_InitStruct.EXTI_LineCmd= ENABLE;
    EXTI_InitStruct.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStruct.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_Init( &EXTI_InitStruct );
}

static void RTC_NVIC_Config( void )
{
    NVIC_InitTypeDef NVIC_InitStructure = {0};

    NVIC_InitStructure.NVIC_IRQChannel = RTCAlarm_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init( &NVIC_InitStructure );
}

/*********************************************************************
 * @fn      RTC_Init
 *
 * @brief   Initializes RTC collection.
 *
 * @return  1 - Init Fail
 *          0 - Init Success
 */
u8 RTC_Init( void )
{
    u8 temp = 0;
    RCC_APB1PeriphClockCmd( RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE );
    PWR_BackupAccessCmd( ENABLE );

    BKP_DeInit();
    RCC_LSICmd(ENABLE);
    while( RCC_GetFlagStatus( RCC_FLAG_LSIRDY ) == RESET && temp < 250 )
    {
        temp++;
        Delay_Ms( 20 );
    }
    if( temp >= 250 )
    {
        return 1;
    }
    RCC_RTCCLKConfig( RCC_RTCCLKSource_LSI );
    RCC_RTCCLKCmd( ENABLE );
    RTC_WaitForLastTask();
    RTC_WaitForSynchro();
    RTC_ITConfig(RTC_IT_ALR, ENABLE);
    RTC_WaitForLastTask();
    RTC_EnterConfigMode();
    RTC_SetPrescaler( 32767 );
    RTC_WaitForLastTask();
    RTC_SetCounter(0);
    RTC_WaitForLastTask();
    RTC_ExitConfigMode();
    BKP_WriteBackupRegister( BKP_DR1, 0XA1A1 );

    RTC_NVIC_Config();

    return 0;
}

static void RTC_Alarm(uint32_t count)
{
    RTC_SetCounter(0);
    RTC_WaitForLastTask();

    RTC_SetAlarm(count);
    RTC_WaitForLastTask();

    EXTI_ClearITPendingBit(EXTI_Line17);
    RTC_ClearITPendingBit(RTC_IT_ALR);

    RTC_ITConfig(RTC_IT_ALR, ENABLE);
    RTC_WaitForLastTask();

    EXTI_ClearITPendingBit(EXTI_Line17);
    RTC_ClearITPendingBit(RTC_IT_ALR);
}





/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main( void )
{
    NVIC_PriorityGroupConfig( NVIC_PriorityGroup_1 );

    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init( 115200 );

    printf("SystemClk:%d\r\n", SystemCoreClock);
    printf( "ChipID:%08x\r\n", DBGMCU_GetCHIPID() );
    printf("Stop Mode Test\r\n");

    RTC_Init();
    RTC17_NVIC_Config();

    RTC_Alarm(10);

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
    printf("\r\n ********** \r\n");
    PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_WFI);
    SystemInit();

    while( 1 )
    {
        Delay_Ms(1000);
        printf("Run in main\r\n");
    }
}

void RTCAlarm_IRQHandler(void) __attribute__((interrupt("WCH-Interrupt-fast")));
/*********************************************************************
 * @fn      RTC_IRQHandler
 *
 * @brief   This function handles RTC Handler.
 *
 * @return  none
 */
void RTCAlarm_IRQHandler( void )
{
    if( RTC_GetITStatus( RTC_IT_ALR ) != RESET ) /* Alarm clock interrupt */
    {
        EXTI_ClearITPendingBit(EXTI_Line17);
        RTC_WaitForLastTask();
        RTC_ClearITPendingBit( RTC_IT_ALR );
        RTC_WaitForLastTask();
    }
}

